{# templates/pyproject.toml.jinja2 #}
# https://python-poetry.org/docs/
# https://packaging.python.org/en/latest/flow/#the-configuration-file

[project]
name = "{{ package_name }}"
version = "0.0.1"
description = "A Lightweight Modular Autonomous True Multi Agent AI Framework."
readme = "README.md"
keywords = ["{{ package_name }}", "AGI", "Agentic AI", "ASI", "superagentx", "agent", "LLM"]

dependencies = [
    "rich>=13.9.2,<14.0.0",
    "websockets>=15.0.1,<16.0.0",
    "python-dotenv>=1.2.1",
    "uvicorn[standard]>=0.38.0",

    # superagentx: GitHub version, extras=all
    "superagentx[all]>=1.0.3",
    "superagentx-handlers>=0.1.6.3",

    # FastAPI
    "fastapi[standard]>=0.115.4,<0.116.0",
]

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"